@@ -9,8 +9,9 @@ use Scalar::Util qw(looks_like_number);
 use Struct::Path 0.80 qw(path);
 
 use App::NDTools::Util qw(chomp_evaled_error);
+use App::NDTools::Slurp qw(s_decode);
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 
 sub MODINFO { "Insert value into structure" }
 
@@ -41,6 +42,7 @@ sub arg_opts {
             }
         },
         'string|value=s' => sub { $self->{OPTS}->{value} = $_[1] },
+        'structure=s' => \$self->{OPTS}->{structure},
     )
 }
 
@@ -61,6 +63,10 @@ sub check_rule {
 sub configure {
     my $self = shift;
 
+    $self->{OPTS}->{value} =
+        s_decode(delete $self->{OPTS}->{structure}, 'JSON')
+            if (defined $self->{OPTS}->{structure});
+
     $self->{OPTS}->{value} =
         $self->load_struct($self->{OPTS}->{file}, $self->{OPTS}->{'file-fmt'})
             if (defined $self->{OPTS}->{file});
@@ -96,6 +102,12 @@ Blame calculation toggle. Enabled by default.
 
 Boolean value to insert.
 
+=item B<--cond> E<lt>pathE<gt>
+
+Apply rule when condition met only. Condition is met when path leads to at
+least one item in the structure. May be used several times (in this case
+conditions are AND'ed).
+
 =item B<--file|-f> E<lt>fileE<gt>
 
 Load inserting value from file.
@@ -124,6 +136,10 @@ Preserve specified substructure. May be used several times.
 
 String to insert.
 
+=item B<--structure> E<lt>JSONE<gt>
+
+JSON structure to insert.
+
 =back
 
 =head1 SEE ALSO
